
/* Chrysalide - Outil d'analyse de fichiers binaires
 * location.c - constitution des localisations d'instructions courantes
 *
 * Copyright (C) 2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "location.h"


#include "id.h"
#include "../../../common/endianness.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : blob  = flux de données à analyser.                          *
*                pos   = position courante dans ce flux. [OUT]                *
*                len   = taille totale des données à analyser.                *
*                sizes = référence pour la taille utilisée.                   *
*                loc   = localisation à fournir. [OUT]                        *
*                                                                             *
*  Description : Lit une localisation d'instruction courante dans l'exécution.*
*                                                                             *
*  Retour      : Bilan de l'opération : true en cas de succès, false sinon.   *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool get_jdwp_location(const bin_t *blob, off_t *pos, off_t len, const jdwp_cmd_vm_id_sizes_reply *sizes, jdwp_location *loc)
{
    if (!read_u8(&loc->tag, blob, pos, len))
        return false;

    if (!get_jdwp_class_id(blob, pos, len, sizes, &loc->class_id))
        return false;

    if (!get_jdwp_method_id(blob, pos, len, sizes, &loc->method_id))
        return false;

    if (!read_u64(&loc->index, blob, pos, len, SRE_BIG))
        return false;

    return true;

}
