
/* Chrysalide - Outil d'analyse de fichiers binaires
 * debug.c - gestion du menu 'Débogage'
 *
 * Copyright (C) 2011-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "debug.h"


#include <string.h>


#include <i18n.h>

#include "../../debug/debugger.h"   /* REMME ! */
#include "../../gtkext/easygtk.h"
#include "../../gui/panels/log.h"



/* Réagit avec le menu "Débogage -> Démarrer/Arrêter". */
static void mcb_debug_start_stop(GtkMenuItem *, GObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : ref      = espace de référencement global.                   *
*                accgroup = groupe d'accélérateurs pour les menus.            *
*                                                                             *
*  Description : Construit le menu "Aide".                                    *
*                                                                             *
*  Retour      : Panneau de menus mis en place.                               *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GtkWidget *build_menu_debug(GObject *ref, GtkAccelGroup *accgroup)
{
    GtkWidget *result;                      /* Support à retourner         */
    GtkWidget *menubar;                     /* Support pour éléments       */
    GtkWidget *submenuitem;                 /* Sous-élément de menu        */

    result = gtk_menu_item_new_with_mnemonic(_("_Debug"));
    gtk_widget_show(result);

    menubar = gtk_menu_new();
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(result), menubar);

    submenuitem = qck_create_menu_item(ref, "mnu_start_stop", _("Start"), G_CALLBACK(mcb_debug_start_stop), ref);
    add_accelerator_to_menu_item(submenuitem, "F9", accgroup);
    gtk_container_add(GTK_CONTAINER(menubar), submenuitem);

    submenuitem = qck_create_menu_separator();
    gtk_container_add(GTK_CONTAINER(menubar), submenuitem);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : menuitem = élément de menu sélectionné.                      *
*                ref      = adresse de l'espace de référencement global.      *
*                                                                             *
*  Description : Réagit avec le menu "Débogage -> Démarrer/Arrêter".          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void mcb_debug_start_stop(GtkMenuItem *menuitem, GObject *ref)
{
    //GLoadedBinary *binary;                  /* Edition courante            */
    GBinaryDebugger *debugger;              /* Module prêt à emploi        */

    //binary = (GLoadedBinary *)g_object_get_data(ref, "current_binary");
    debugger = NULL;//get_main_debugger_for_binary(get_current_openida_project(), binary);

    if (debugger == NULL)
    {
        log_simple_message(LMT_ERROR, _("No available debug module for this binary !"));
        return;
    }

    if (strcmp(gtk_menu_item_get_label(menuitem), _("Start")) == 0)
    {
        if (!g_binary_debugger_attach(debugger))
        {
            log_simple_message(LMT_ERROR, _("Failed to initialize the debugger."));
            return;
        }

        gtk_menu_item_set_label(menuitem, _("Stop"));

    }
    else
    {
        /* TODO */

        gtk_menu_item_set_label(menuitem, _("Start"));

    }

}
